package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.dto.*;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.enums.StatisticsType;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.VariableService;
import com.syc.slm.slmbi.vo.*;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 变量
 *
 * @author : lin
 * @date : 2021-03-25 18:36
 **/
@Service
public class VariableServiceImpl implements VariableService {
    @Override
    public IPage<VariableVo> getVariableList(VariableDTO variableDTO,String appId,String token,String uri) {
        IPage<VariableVo> page = new Page<>(variableDTO.getCurrent(),variableDTO.getSize());
        Map<String,String> p = Maps.newHashMap();
        p.put("appId",appId);
        p.put("token",token);
        p.put("from",variableDTO.getSourceId());
        p.put("varType",null);
        p.put("type",null);
        p.put("equipmentId",variableDTO.getDeviceId());
        p.put("customerId",null);
        p.put("name",variableDTO.getVariableName());
        p.put("desc",variableDTO.getDescription());
        p.put("unit",null);
        p.put("keyword",null);
        p.put("isCustomer",null);
        p.put("current",variableDTO.getCurrent()+"");
        p.put("size",variableDTO.getSize()+"");

        R<Map<String, Object>> result = RestTemplateUtils.get(uri, new TypeToken<R<Map<String, Object>>>() {
        }.getType(), p);
        Map<String, Object> detach = result.detach();
        if (CollectionUtils.isNotEmpty(detach)) {
            List<VariableVo> vos = SlmConstants.gsonLowerCaseWithUnderscores.fromJson(SlmConstants.gson.toJson(detach.get("datas")),
                                                                    new TypeToken<List<VariableVo>>() {
                                                                    }.getType());
            page.setTotal((new BigDecimal(detach.get("total") + "")).longValue());
            page.setRecords(vos);
        }
        return page;
    }


    @Override
    public List<VariableSourceVo> getVariableSourceList() {
        List<VariableSourceVo> vos = Lists.newArrayList();
        VariableSourceVo vo = new VariableSourceVo();
        vo.setSourceId("6");
        vo.setSourceName("请选择变量来源");
        VariableSourceVo vo1 = new VariableSourceVo();
        vo1.setSourceId("1");
        vo1.setSourceName("基础采集点");
        VariableSourceVo vo2 = new VariableSourceVo();
        vo2.setSourceId("4");
        vo2.setSourceName("配置变量/dc公式");
        vos.add(vo);
        vos.add(vo1);
        vos.add(vo2);
        return vos;
    }

    @Override
    public IPage<VariableDeviceVo> getVariableDeviceList(String token,String appId,String uri, QueryPageDTO queryPageDTO) {

        IPage<VariableDeviceVo> page = new Page<>(queryPageDTO.getCurrent(), queryPageDTO.getSize());
        Map<String, String> p = Maps.newHashMap();
        p.put("appId", appId);
        p.put("token", token);
        p.put("equipment_info_id", null);
        p.put("customer_id", null);
        p.put("q_name", null);
        p.put("search_code", null);

        p.put("search_model", null);
        p.put("warranty_status", null);
        p.put("page_num", queryPageDTO.getCurrent() + "");
        p.put("page_size", queryPageDTO.getSize() + "");
        R<Map<String, Object>> result = RestTemplateUtils.get(uri, new TypeToken<R<Map<String, Object>>>() {
        }.getType(), p);
        Map<String, Object> detach = result.detach();
        if (CollectionUtils.isNotEmpty(detach)) {
            List<VariableDeviceVo> vos = SlmConstants.gsonLowerCaseWithUnderscores.fromJson(SlmConstants.gson.toJson(detach.get("result")),
                                                                    new TypeToken<List<VariableDeviceVo>>() {
                                                                    }.getType());
            page.setTotal((new BigDecimal(detach.get("total") + "")).longValue());
            page.setRecords(vos);
        }

        return page;
    }

    @Override
    public Map<String,List<Object>> listVariableAllDataListByTimeInterval(@NonNull String uri,
        VariableAllDataDTO variableDTO,CurrentUser user) {
        if(StringUtils.isBlank(uri)){
            throw new SysException("请传入uri");
        }
        if(CollectionUtils.isEmpty(variableDTO.getVariableSearchList())){
            throw new SysException("请传入变量");
        }else{
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getDeviceId()))){
                throw new SysException("设备ID不能为空");
            }
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableId()))){
                throw new SysException("变量ID不能为空");
            }
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableName()))){
                throw new SysException("变量名称不能为空");
            }
        }
        if(StringUtils.isBlank(variableDTO.getEndTime())){
            throw new SysException("请传入结束时间");
        }
        if(StringUtils.isBlank(variableDTO.getStartTime())){
            throw new SysException("请传入开始时间");
        }
        // Map<String, String> uriVariables =SlmConstants.gson.fromJson(SlmConstants.gson.toJson(variableDTO),new TypeToken<Map<String,String>>(){}.getType());
        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        R<Map<String,List<Object>>> result = RestTemplateUtils.post(uri,variableDTO,new TypeToken<R<Map<String,List<Object>>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    @Override
    public  Map<String,Object> listVariableHistoryDataListByCode(@NonNull String uri,
        VariableHistoryDataDTO variableDTO,CurrentUser user) {
        if(StringUtils.isBlank(uri)){
            throw new SysException("请传入uri");
        }
        if(CollectionUtils.isEmpty(variableDTO.getVariableSearchList())){
            throw new SysException("请传入变量");
        }else{
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getDeviceId()))){
                throw new SysException("设备ID不能为空");
            }
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableId()))){
                throw new SysException("变量ID不能为空");
            }
            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableName()))){
                throw new SysException("变量名称不能为空");
            }
        }

        if(StringUtils.isBlank(variableDTO.getEndTime())){
            throw new SysException("请传入结束时间");
        }
        if(StringUtils.isBlank(variableDTO.getStartTime())){
            throw new SysException("请传入开始时间");
        }

        if(StringUtils.isBlank(variableDTO.getStatisticsType())){
                throw new SysException("请传入统计类型");
        }else{
            if(!StatisticsType.SUM.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.AVG.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.MAX.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.MIN.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.MIDDLE.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.ORIGINAL.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.COUNT.value.equals(variableDTO.getStatisticsType()) &&
                        !StatisticsType.DUPLICATE_COUNT.value.equals(variableDTO.getStatisticsType())){
                throw new SysException("没有对应匹配的统计类型");
            }
        }
       // Map<String, String> uriVariables =SlmConstants.gson.fromJson(SlmConstants.gson.toJson(variableDTO),new TypeToken<Map<String,String>>(){}.getType());
        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());

        R<Map<String,Object>> result = RestTemplateUtils.post(uri,variableDTO,new TypeToken<R<Map<String,Object>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void send(VariableSendDTO sendDTO, CurrentUser currentUser,String uri) {
        if (StringUtils.isBlank(sendDTO.getAccessType())) {
            throw new SysException("请传入访问类型");
        }

        if(AccessType.PRIVATE.value.equals(sendDTO.getAccessType())){
            if (StringUtils.isBlank(sendDTO.getUserId())) {
                throw new SysException("访问类型为私有，请传入创建人");
            }
            if(currentUser.getUid().equals(sendDTO.getUserId())){
                throw new SysException("没有访问权限");
            }
        }else if(AccessType.CUSTOM.value.equals(sendDTO.getAccessType())) {
            if(CollectionUtils.isNotEmpty(sendDTO.getPermissions())){
                Map<String, List<CustomPermissionDTO>> maps =
                    sendDTO.getPermissions().stream().collect(Collectors.groupingBy(CustomPermissionDTO::getBizType));
                List<CustomPermissionDTO> departments = maps.get("department");
                List<CustomPermissionDTO> positions = maps.get("position");
                List<CustomPermissionDTO> users = maps.get("user");
                boolean flag=true;
                if(CollectionUtils.isNotEmpty(users)){
                    if(users.stream().noneMatch(x->currentUser.getUid().equals(x.getBizId()))){
                        flag=false;
                    }
                }
                if(CollectionUtils.isNotEmpty(departments) || CollectionUtils.isNotEmpty(positions)) {
                    R<Map<String, String>> result = setDeptAndRole(currentUser, uri);
                    Map<String, String> detach = result.detach();
                    String officeIds = detach.get("deptIds");
                    String roleIds = detach.get("roleIds");
                    if (flag) {
                        if (CollectionUtils.isNotEmpty(departments)) {
                            if (departments.stream()
                                .map(CustomPermissionDTO::getBizId)
                                .anyMatch(officeIds::contains)) {
                                flag = false;
                            }
                        }
                    }
                    if (flag) {
                        if (CollectionUtils.isNotEmpty(positions)) {
                            if (positions.stream().map(CustomPermissionDTO::getBizId).anyMatch(roleIds::contains)) {
                                flag = false;
                            }
                        }
                    }
                    if(flag){
                        throw new SysException("没有访问权限");
                    }
                }
            }

        }

        Map<String, Object> commandMap = sendDTO.getCommandMap();
        if(CollectionUtils.isEmpty(commandMap)){
            throw new SysException("变量值必传");
        }

        if(commandMap.keySet().stream().anyMatch(StringUtils::isBlank)){
            throw new SysException("变量id 不能为空");
        }
        if(commandMap.values().stream().anyMatch(ObjectUtils::isEmpty)){
            throw new SysException("变量对应得value 值不能为空");
        }

        SendDeviceCommandsDTO dto = new SendDeviceCommandsDTO();
        dto.setCommandMap(commandMap);
        dto.setToken(currentUser.getToken());
        dto.setAppId(currentUser.getAppId());
        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId", currentUser.getAppId());
        R<Object> result = RestTemplateUtils.post(uri, null, dto, new TypeToken<R<Object>>() {}.getType(), uriVariables);
        result.detach();

    }

    @Override
    public Map<String, Object> status(String uri, String deviceId,String variableId, String variableCode,CurrentUser user) {
        if(StringUtils.isBlank(variableCode)){
            throw new SysException("变量code不能为空");
        }

        if(StringUtils.isBlank(deviceId)){
            throw new SysException("设备id不能为空");
        }
        Map<String,String> uriVariables =Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        uriVariables.put("deviceId",deviceId);
        uriVariables.put("variableId",variableId);
        uriVariables.put("variableCode",variableCode);
        R<Map<String,Object>> result = RestTemplateUtils.get(uri,new TypeToken<R<Map<String,Object>>>() {}.getType(), uriVariables);
        return  result.detach();
    }

    @Override
    public Map<String, Object> historyStatus(String uri,
        String deviceId,
        String variableCode,
        String startTime,
        String endTime,
        CurrentUser currentUser) {


        if(StringUtils.isBlank(variableCode)){
            throw new SysException("变量code不能为空");
        }

        if(StringUtils.isBlank(deviceId)){
            throw new SysException("设备id不能为空");
        }
        if(StringUtils.isBlank(startTime)){
            throw new SysException("开始时间不能为空");
        }

        if(StringUtils.isBlank(endTime)){
            throw new SysException("结束时间不能为空");
        }
        Map<String,String> uriVariables =Maps.newHashMap();
        uriVariables.put("appId",currentUser.getAppId());
        uriVariables.put("deviceId",deviceId);
        uriVariables.put("variableCode",variableCode);
        uriVariables.put("startTime",startTime);
        uriVariables.put("endTime",endTime);
        R<Map<String,Object>> result = RestTemplateUtils.get(uri,new TypeToken<R<Map<String,Object>>>() {}.getType(), uriVariables);
        return  result.detach();
    }

    @Override
    public Map<String,List<Object>> listRealTimeVariableData(@NonNull String uri, RealTimeVariableDataDTO variableDTO, CurrentUser user) {
        this.judgeSearchList(variableDTO.getVariableSearchList());
        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        R<Map<String,List<Object>>> result = RestTemplateUtils.post(uri,variableDTO,new TypeToken<R<Map<String,List<Object>>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    @Override
    public Map<String, Map<String, Object>> listVariableStatusRealTimeData(@NonNull String uri, List<VariableStatusRealTimeDataDTO> statusRealTimeDataList, CurrentUser user) {
        if(CollectionUtils.isEmpty(statusRealTimeDataList)){
            throw new SysException("请传入变量");
        }else{
            if(statusRealTimeDataList.stream().anyMatch(x -> StringUtils.isBlank(x.getDeviceId()))){
                throw new SysException("设备ID不能为空");
            }
            if(statusRealTimeDataList.stream().anyMatch(x -> StringUtils.isBlank(x.getVariableId()))){
                throw new SysException("变量Id不能为空");
            }
            if(statusRealTimeDataList.stream().anyMatch(x -> StringUtils.isBlank(x.getVariableCode()))){
                throw new SysException("变量Code不能为空");
            }
        }
        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        R<Map<String, Map<String, Object>>> result = RestTemplateUtils.post(uri,statusRealTimeDataList,new TypeToken<R<Map<String, Map<String, Object>>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    private void judgeSearchList(List<VariableSearchDTO> variableSearchList){
        if(CollectionUtils.isEmpty(variableSearchList)){
            throw new SysException("请传入变量");
        }else{
            if(variableSearchList.stream().anyMatch(x -> StringUtils.isBlank(x.getDeviceId()))){
                throw new SysException("设备ID不能为空");
            }
            if(variableSearchList.stream().anyMatch(x -> StringUtils.isBlank(x.getVariableId()))){
                throw new SysException("变量ID不能为空");
            }
            if(variableSearchList.stream().anyMatch(x -> StringUtils.isBlank(x.getVariableName()))){
                throw new SysException("变量名称不能为空");
            }
        }
    }

    //    @Override
//    public Map<String,List<Map<String,Object>>> getVariableHistoryCurve(VariableHistoryCurveDataDTO variableDTO, CurrentUser user,String uri) {
//        if(CollectionUtils.isEmpty(variableDTO.getVariableSearchList())){
//            throw new SysException("请传入变量");
//        }else{
//            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getDeviceId()))){
//                throw new SysException("设备ID不能为空");
//            }
//            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableId()))){
//                throw new SysException("变量ID不能为空");
//            }
//            if(variableDTO.getVariableSearchList().stream().anyMatch(x -> StringUtils.isBlank(x.getVariableName()))){
//                throw new SysException("变量名称不能为空");
//            }
//        }
//        if(StringUtils.isBlank(variableDTO.getEndTime())){
//            throw new SysException("请传入结束时间");
//        }
//        if(StringUtils.isBlank(variableDTO.getStartTime())){
//            throw new SysException("请传入开始时间");
//        }
//
//
//        //Map<String, String> uriVariables =SlmConstants.gson.fromJson(SlmConstants.gson.toJson(variableDTO),new TypeToken<Map<String,String>>(){}.getType());
//        Map<String, String> uriVariables = Maps.newHashMap();
//        uriVariables.put("appId",user.getAppId());
//
//        R<Map<String,List<Map<String,Object>>>> result = RestTemplateUtils.post(uri,variableDTO,new TypeToken<R<Map<String,List<Map<String,Object>>>>>() {}.getType(), uriVariables);
//        return result.detach();
//    }

    private  R<Map<String,String>> setDeptAndRole(CurrentUser user,String uri){
        Map<String,String> uriVariables =Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        return  RestTemplateUtils.get(uri,new TypeToken<R<Map<String,String>>>() {}.getType(), uriVariables);

    }

}
